@@ -42,6 +42,7 @@ class SruRequest:
     :ivar error_messages: list of string with the error messages
     :ivar are_more_results_available: bool indicating if more records than the limit is available
     :ivar records: list of :class:`almasru.client.SruRecord`
+    :ivar base_url: base url of the SRU server
 
     :example:
 
@@ -49,13 +50,14 @@ class SruRequest:
 
     """
 
-    def __init__(self, query: str, limit: int = 10) -> None:
+    def __init__(self, query: str, limit: int = 10, client: 'SruClient' = None) -> None:
         self.query = query
         self.limit = limit
         self.error = False
         self.error_messages = []
         self.are_more_results_available = False
         self.records = []
+        self.base_url = client.base_url if client is not None else SruClient.base_url
         self._fetch_records()
 
     def __hash__(self) -> int:
@@ -96,7 +98,7 @@ class SruRequest:
                 content = f.read()
         else:
             params = self._build_sru_params(self.query, start_record, maximum_records)
-            r = requests.get(SruClient.base_url, params=params)
+            r = requests.get(self.base_url, params=params)
             if r.ok is True:
                 logging.info(f'SRU data fetched: {r.url}')
                 content = r.content
@@ -117,6 +119,8 @@ class SruRequest:
             logging.error(self.error_messages[-1])
             is_error = True
 
+            xml = etree.fromstring(content, parser=SruClient.parser)
+
         return xml, is_error
 
     def _fetch_records(self) -> None:
@@ -224,7 +228,7 @@ class SruClient:
         :return: :class:`almasru.client.SruRequest`
         """
         if f'{query}__{limit}' not in self.requests:
-            req = SruRequest(query, limit)
+            req = SruRequest(query, limit, self)
             self.requests[f'{query}__{limit}'] = req
 
         return self.requests[f'{query}__{limit}']
@@ -311,11 +315,12 @@ class SruRecord:
             return None
 
     @check_error
-    def get_035_fields(self, var: bool = False) -> Set[str]:
-        """get_035_fields(self, var: bool = False) -> Set[str]
+    def get_035_fields(self, var: bool = False, slsp_only: bool = False) -> Set[str]:
+        """get_035_fields(self, var: bool = False, slsp_only: bool = False) -> Set[str]
         Return a set of system numbers of 035 fields
 
         :param var: Parameters: flag indicating if truncated system numbers should be tested
+        :param slsp_only: Parameters: flag indicating if only slsp system numbers should be returned
 
         :return: Set of strings with the system IDs of 035 fields
         """
@@ -333,6 +338,11 @@ class SruRecord:
 
             other_sys_ids.update(var_sys_id)
 
+        if slsp_only is True:
+            # return only SLSP related system ids
+            prefixes = '|'.join(['RERO', 'IDSBB', 'IDSLU', 'IDSSG', 'NEBIS', 'SBT', 'ALEX', 'ABN', 'swissbib'])
+            other_sys_ids = {sys_num for sys_num in other_sys_ids
+                             if re.match(r'^\((?:{})\).+'.format(prefixes), sys_num)}
         return other_sys_ids
 
     @check_error
@@ -424,7 +434,8 @@ class SruRecord:
             return self._child_rec_num_sys
 
         # Fetch the system numbers, all 035 + 001 should be tested
-        sys_numbers_to_test = self.get_035_fields()
+        sys_numbers_to_test = self.get_035_fields(slsp_only=True)
+
         sys_numbers_to_test.add(self.mms_id)
 
         fields_related_records = []
@@ -456,8 +467,8 @@ class SruRecord:
             if len(temp_records) == 0:
                 continue
 
-            # Use sets to avoid duplicated records
-            records.update(temp_records)
+            # Use to ignore records linked with 035 that are not real children
+            temp_records_duplicates = set()
 
             for record in temp_records:
                 temp_fields = record.data.xpath(f'./m:datafield/m:subfield[contains(text(), "{sys_num}")]',
@@ -469,18 +480,21 @@ class SruRecord:
                         tag = field.getparent().attrib['tag']
 
                         if tag == '035':
+                            temp_records_duplicates.add(record)
                             self.warning = True
                             self.warning_messages.append(f'More than one record with same 035, {self.mms_id} '
                                                          f'and {record.mms_id} are probably duplicated records')
-
-                        fields_related_records.append({'child_MMS_ID': record.mms_id,
-                                                       'field': f'{tag}${code}',
-                                                       'content': field.text})
+                        else:
+                            fields_related_records.append({'child_MMS_ID': record.mms_id,
+                                                           'field': f'{tag}${code}',
+                                                           'content': field.text})
                 else:
                     logging.warning(f'{repr(self)} record {repr(record)} found with SRU but no linking field')
                     fields_related_records.append({'child_MMS_ID': record.mms_id,
                                                    'field': f'UNKNOWN',
                                                    'content': sys_num})
+            # Use sets to avoid duplicated records
+            records.update(set(temp_records) - temp_records_duplicates)
 
         if len(records) == 0:
             related_record_found = False
@@ -539,8 +553,8 @@ class SruRecord:
             if len(temp_records) == 0:
                 continue
 
-            # Use sets to avoid duplicated records
-            records.update(temp_records)
+            # Use to ignore records linked with 020 or 022 that are not real children
+            temp_records_duplicates = set()
 
             for record in temp_records:
                 temp_fields = record.data.xpath(f'./m:datafield/m:subfield[contains(text(), "{std_num}")]',
@@ -550,14 +564,21 @@ class SruRecord:
                     for field in temp_fields:
                         code = field.attrib['code']
                         tag = field.getparent().attrib['tag']
-                        fields_related_records.append({'child_MMS_ID': record.mms_id,
-                                                       'field': f'{tag}${code}',
-                                                       'content': field.text})
+                        if tag in ['020', '022']:
+                            self.warning = True
+                            self.warning_messages.append(f'More than one record with same {tag}, {self.mms_id} '
+                                                         f'and {record.mms_id} are probably duplicated records')
+                        else:
+                            fields_related_records.append({'child_MMS_ID': record.mms_id,
+                                                           'field': f'{tag}${code}',
+                                                           'content': field.text})
                 else:
                     logging.warning(f'{repr(self)} record {repr(record)} found with SRU but no linking field')
                     fields_related_records.append({'child_MMS_ID': record.mms_id,
                                                    'field': f'UNKNOWN',
                                                    'content': std_num})
+            # Use sets to avoid duplicated records
+            records.update(set(temp_records) - temp_records_duplicates)
 
         if len(records) == 0:
             related_record_found = False
@@ -587,7 +608,7 @@ class SruRecord:
             return self._parent_rec
 
         # List of datafields to check
-        fields_to_test = ['773', '774', '776', '777', '780', '785', '786', '800', '810', '811', '830']
+        fields_to_test = ['773', '774', '776', '777', '780', '785', '786', '787', '800', '810', '811', '830']
 
         fields_related_records = []
         records = set()
@@ -719,22 +740,110 @@ class SruRecord:
         return list_izs
 
     @check_error
-    def is_removable(self) -> Tuple[bool, str]:
-        """is_removable(self) -> Tuple[bool, str]
-        Check if a record is safe to be removed
+    def get_child_analytical_records(self) -> List['SruRecord']:
+        """get_child_analytical_records(self) -> List['SruRecord']
+        Get the list of analytical records. Check only existing 773 fields and not leader.
 
-        The method checks related records and inventory in other IZ.
-        1. Test if the record has 852 fields. It would indicate existing holding in any IZ
-        2. Test if the record is target of 8xx or 773 fields of other records
-        3. Test if the record has a 773 field targeting a record with inventory
-
-        :return: tuple containing bool indicating if the record can be safely removed and a message.
+        :return: list of :class:`almasru.client.SruRecord`
         """
+        children_analysis = self.get_child_rec_sys_num()
+        analytical_children = []
+        for link in children_analysis['fields_related_records']:
+            if link['field'] == '773$w':
+                analytical_children.append(SruRecord(link['child_MMS_ID']))
+
+        return analytical_children
+
+    @check_error
+    def get_reasons_preventing_deletion(self, removable_rec_mms_id: Optional[List[str]] = None) -> List[str]:
+        """analyse(self, removable_rec_mms_id: Optional[List[str]] = None) => List[str]
+        Analyse the record to find reasons preventing deletion
+
+        :param removable_rec_mms_id: list of MMS_ID of records that are safe to be removed. This is used to avoid
+            records of the list currently being processed to be considered as preventing deletion.
+
+        :return: list of string with the reasons preventing deletion
+        """
+        # Set default value for removable_rec_mms_id
+        if removable_rec_mms_id is None:
+            removable_rec_mms_id = []
+        messages = []
+
         # Check if record used by other IZ and has holdings
         list_izs = self.get_iz_using_rec()
         if len(list_izs) > 0:
-            logging.warning(f'{repr(self)} cannot be deleted: record used in other IZs: {", ".join(list_izs)}')
-            return False, 'Record used in other IZ'
+            logging.info(f'{repr(self)} used in IZs: {", ".join(list_izs)}')
+            messages.append(f'Used in IZs: {", ".join(list_izs)}')
+
+        # Filter the list with the list of removable records.
+        analytical_children = [rec for rec in self.get_child_analytical_records()
+                               if rec.mms_id not in removable_rec_mms_id]
+
+        if len(analytical_children) > 0:
+            logging.info(f'{repr(self)} has at least one child '
+                         f'analytical record: {repr(analytical_children[0])}')
+            messages.append(f'Has child analytical records: {"|".join([rec.mms_id for rec in analytical_children])}')
+
+        # Check if the record has children with inventory. The child must have be linked with a 8xx field
+        # or 773. Normally analytical records linked with 773 should not have holdings anyway.
+        children = self.get_child_removable_candidate_rec()
+        for rec in children:
+            used_by_iz = rec.get_iz_using_rec()
+
+            if len(used_by_iz) > 0:
+                logging.info(f'{repr(self)} has at least a child record with '
+                             f'inventory: {repr(rec)} in '
+                             f'{", ".join(used_by_iz)}')
+                messages.append(f'Has child preventing deletion record with inventory: {rec.mms_id}')
+
+        # Get the parent records. Only parent that are target of 773 field prevent deletion of the child
+        # if the parent has inventory
+        parents = self.get_parent_removable_candidate_rec()
+        for rec in parents:
+            used_by_iz = rec.get_iz_using_rec()
+            if len(used_by_iz) > 0:
+                logging.info(f'{repr(self)} has at least a parent record with '
+                             f'inventory: {repr(rec)} in '
+                             f'{", ".join(used_by_iz)}')
+                messages.append(f'Has parent record preventing deletion with inventory: {rec.mms_id}')
+
+        return messages
+
+    @check_error
+    def is_removable(self, removable_rec_mms_id: Optional[List[str]] = None) -> Tuple[bool, str]:
+        """is_removable(self, removable_rec_mms_id: Optional[List[str]] = None) -> Tuple[bool, str]
+        Check if a record is safe to be removed
+
+        :param removable_rec_mms_id: list of MMS_ID of records that are safe to be removed. This is used to avoid
+            records of the list currently being processed to be considered as preventing deletion.
+
+        The method checks related records and inventory in other IZ.
+        1. Test if the record has 852 fields. It would indicate existing holding in any IZ
+        2. Test if record has analytical records children (children linked with 773 field)
+        3. Test if the record is target of 8xx or 773 fields of other records
+        4. Test if the record has a 773 field targeting a record with inventory
+
+        :return: tuple containing bool indicating if the record can be safely removed and a message.
+        """
+        # Set default value for removable_rec_mms_id
+        if removable_rec_mms_id is None:
+            removable_rec_mms_id = []
+
+        # Check if record used by other IZ and has holdings
+        list_izs = self.get_iz_using_rec()
+        if len(list_izs) > 0:
+            logging.warning(f'{repr(self)} cannot be deleted: record used in IZs: {", ".join(list_izs)}')
+            return False, 'Record used in at least one IZ'
+
+        # No deletion if records has analytical records among children
+        # Filter the list with the list of removable records.
+        analytical_children = [rec for rec in self.get_child_analytical_records()
+                               if rec.mms_id not in removable_rec_mms_id]
+
+        if len(analytical_children) > 0:
+            logging.warning(f'{repr(self)} cannot be deleted: record has at least one child '
+                            f'analytical record: {repr(analytical_children[0])}')
+            return False, 'Has analytical record as child'
 
         # Check if the record has children with inventory. The child must have be linked with a 8xx field
         # or 773. Normally analytical records linked with 773 should not have holdings anyway.
@@ -778,6 +887,7 @@ class SruRecord:
         children_mms_id = set([field['child_MMS_ID'] for field in children['fields_related_records']
                               if field['field'][:3] in ['773', '800', '810', '811', '830']])
 
+        # NOTE: le 773 empêche la suppression: code à corriger
         return [SruRecord(mms_id) for mms_id in children_mms_id]
 
     @check_error
